/*
 * pblfixup.c - Fixup a Q;Q; style block checksum.
 *
 * Copyright 2006 Jonathan McDowell <noodles@earth.li>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define BUFSIZE 16384

int main(int argc, char *argv[])
{
	uint32_t r0, r2;
	size_t len;
	unsigned char buf[BUFSIZE];
	int i;

	len = read(STDIN_FILENO, buf, BUFSIZE);

	if (len == BUFSIZE) {
		fprintf(stderr, "Block too big.\n");
		exit(EXIT_FAILURE);
	}

	if ((buf[8] + buf[9] * 256) != len) {
		len = buf[8] + buf[9] * 256;
		fprintf(stderr, "Overridding len to %d\n", len);
	}

	r0 = r2 = 0;
	for (i = 0; i < len; i++) {
		r0 += buf[i];
		r0 = r0 & 0xFF;
		r2 += r0;
		r2 = r2 & 0xFF;
	} 
	fprintf(stderr, "r0 = 0x%04X, r2 = 0x%04X\n", r0, r2);

	/* 
	 * X = -r2 + (L-n).r0
	 * Y =  r2 - (L-n+1).r0
	 */
	fprintf(stderr, "Fixup bytes for offset 6 = 0x%02X, 7 = 0x%02X\n",
			(-r2 + (len - 7) * r0) & 0xFF,
			(r2 - (len - 6) * r0) & 0xFF);

	buf[6] = (-r2 + (len - 7) * r0) & 0xFF;
	buf[7] = (r2 - (len - 6) * r0) & 0xFF;

	write(STDOUT_FILENO, buf, len);

	return EXIT_SUCCESS;
}
